@@ -42,10 +42,12 @@
 #include <string>
 #include <vector>
 
-#include <google/protobuf/descriptor.h>
-#include <google/protobuf/wire_format_lite.h>
-#include <google/protobuf/util/type_resolver.h>
-#include "conformance.pb.h"
+#include "google/protobuf/descriptor.h"
+#include "google/protobuf/util/type_resolver.h"
+#include "absl/container/btree_set.h"
+#include "absl/container/flat_hash_set.h"
+#include "conformance/conformance.pb.h"
+#include "google/protobuf/wire_format_lite.h"
 
 namespace conformance {
 class ConformanceRequest;
@@ -88,10 +90,12 @@ class ForkPipeRunner : public ConformanceTestRunner {
                  const std::vector<ConformanceTestSuite*>& suites);
 
   ForkPipeRunner(const std::string& executable,
-                 const std::vector<std::string>& executable_args)
+                 const std::vector<std::string>& executable_args,
+                 bool performance)
       : child_pid_(-1),
         executable_(executable),
-        executable_args_(executable_args) {}
+        executable_args_(executable_args),
+        performance_(performance) {}
 
   explicit ForkPipeRunner(const std::string& executable)
       : child_pid_(-1), executable_(executable) {}
@@ -114,6 +118,7 @@ class ForkPipeRunner : public ConformanceTestRunner {
   pid_t child_pid_;
   std::string executable_;
   const std::vector<std::string> executable_args_;
+  bool performance_;
   std::string current_test_name_;
 };
 
@@ -148,10 +153,12 @@ class ConformanceTestSuite {
  public:
   ConformanceTestSuite()
       : verbose_(false),
+        performance_(false),
         enforce_recommended_(false),
         failure_list_flag_name_("--failure_list") {}
   virtual ~ConformanceTestSuite() {}
 
+  void SetPerformance(bool performance) { performance_ = performance; }
   void SetVerbose(bool verbose) { verbose_ = verbose; }
 
   // Whether to require the testee to pass RECOMMENDED tests. By default failing
@@ -256,8 +263,6 @@ class ConformanceTestSuite {
     std::string test_name_;
   };
 
-  bool CheckSetEmpty(const std::set<std::string>& set_to_check,
-                     const std::string& write_to_file, const std::string& msg);
   std::string WireFormatToString(conformance::WireFormat wire_format);
 
   // Parse payload in the response to the given message. Returns true on
@@ -272,11 +277,17 @@ class ConformanceTestSuite {
                       const conformance::ConformanceResponse& response,
                       bool need_report_success, bool require_same_wire_format);
 
+  void TruncateDebugPayload(std::string* payload);
+  const conformance::ConformanceRequest TruncateRequest(
+      const conformance::ConformanceRequest& request);
+  const conformance::ConformanceResponse TruncateResponse(
+      const conformance::ConformanceResponse& response);
+
   void ReportSuccess(const std::string& test_name);
   void ReportFailure(const std::string& test_name, ConformanceLevel level,
                      const conformance::ConformanceRequest& request,
                      const conformance::ConformanceResponse& response,
-                     const char* fmt, ...);
+                     absl::string_view message);
   void ReportSkip(const std::string& test_name,
                   const conformance::ConformanceRequest& request,
                   const conformance::ConformanceResponse& response);
@@ -299,6 +310,7 @@ class ConformanceTestSuite {
   int successes_;
   int expected_failures_;
   bool verbose_;
+  bool performance_;
   bool enforce_recommended_;
   std::string output_;
   std::string output_dir_;
@@ -307,20 +319,20 @@ class ConformanceTestSuite {
 
   // The set of test names that are expected to fail in this run, but haven't
   // failed yet.
-  std::set<std::string> expected_to_fail_;
+  absl::btree_set<std::string> expected_to_fail_;
 
   // The set of test names that have been run.  Used to ensure that there are no
   // duplicate names in the suite.
-  std::set<std::string> test_names_;
+  absl::flat_hash_set<std::string> test_names_;
 
   // The set of tests that failed, but weren't expected to.
-  std::set<std::string> unexpected_failing_tests_;
+  absl::btree_set<std::string> unexpected_failing_tests_;
 
   // The set of tests that succeeded, but weren't expected to.
-  std::set<std::string> unexpected_succeeding_tests_;
+  absl::btree_set<std::string> unexpected_succeeding_tests_;
 
   // The set of tests that the testee opted out of;
-  std::set<std::string> skipped_;
+  absl::btree_set<std::string> skipped_;
 };
 
 }  // namespace protobuf
