@@ -109,6 +109,11 @@ __attribute__((objc_subclassing_restricted))
  **/
 - (void)flush;
 
+/**
+ * @return The number of bytes written out. Includes bytes not yet flused.
+ **/
+- (size_t)bytesWritten;
+
 /**
  * Write the raw byte out.
  *
