@@ -62,14 +62,18 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 /**
  * Describes a proto message.
  **/
+__attribute__((objc_subclassing_restricted))
 @interface GPBDescriptor : NSObject<NSCopying>
 
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
+
 /** Name of the message. */
 @property(nonatomic, readonly, copy) NSString *name;
 /** Fields declared in the message. */
-@property(nonatomic, readonly, strong, nullable) NSArray<GPBFieldDescriptor*> *fields;
+@property(nonatomic, readonly, strong, nullable) NSArray<GPBFieldDescriptor *> *fields;
 /** Oneofs declared in the message. */
-@property(nonatomic, readonly, strong, nullable) NSArray<GPBOneofDescriptor*> *oneofs;
+@property(nonatomic, readonly, strong, nullable) NSArray<GPBOneofDescriptor *> *oneofs;
 /** Extension range declared for the message. */
 @property(nonatomic, readonly, nullable) const GPBExtensionRange *extensionRanges;
 /** Number of extension ranges declared for the message. */
@@ -121,25 +125,42 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 /**
  * Describes a proto file.
  **/
-@interface GPBFileDescriptor : NSObject
+__attribute__((objc_subclassing_restricted))
+@interface GPBFileDescriptor : NSObject<NSCopying>
+
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
 
 /** The package declared in the proto file. */
 @property(nonatomic, readonly, copy) NSString *package;
 /** The objc prefix declared in the proto file. */
 @property(nonatomic, readonly, copy, nullable) NSString *objcPrefix;
-/** The syntax of the proto file. */
-@property(nonatomic, readonly) GPBFileSyntax syntax;
+/**
+ * The syntax of the proto file.
+ *
+ * This should not be used for making decisions about support
+ * features/behaviors, what proto2 vs. proto3 syntax has meant has evolved over
+ * time, and not more specific methods on the descriptors should be used
+ * instead.
+ */
+@property(nonatomic, readonly) GPBFileSyntax syntax
+    __attribute__((deprecated("Syntax is not a good way to decide things about behaviors.")));
 
 @end
 
 /**
  * Describes a oneof field.
  **/
-@interface GPBOneofDescriptor : NSObject
+__attribute__((objc_subclassing_restricted))
+@interface GPBOneofDescriptor : NSObject<NSCopying>
+
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
+
 /** Name of the oneof field. */
 @property(nonatomic, readonly) NSString *name;
 /** Fields declared in the oneof. */
-@property(nonatomic, readonly) NSArray<GPBFieldDescriptor*> *fields;
+@property(nonatomic, readonly) NSArray<GPBFieldDescriptor *> *fields;
 
 /**
  * Gets the field for the given number.
@@ -164,7 +185,11 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 /**
  * Describes a proto field.
  **/
-@interface GPBFieldDescriptor : NSObject
+__attribute__((objc_subclassing_restricted))
+@interface GPBFieldDescriptor : NSObject<NSCopying>
+
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
 
 /** Name of the field. */
 @property(nonatomic, readonly, copy) NSString *name;
@@ -182,7 +207,7 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 @property(nonatomic, readonly, getter=isOptional) BOOL optional;
 /** Type of field (single, repeated, map). */
 @property(nonatomic, readonly) GPBFieldType fieldType;
-/** Type of the key if the field is a map. The value's type is -fieldType. */
+/** Type of the key if the field is a map. The value's type is -dataType. */
 @property(nonatomic, readonly) GPBDataType mapKeyDataType;
 /** Whether the field is packable. */
 @property(nonatomic, readonly, getter=isPackable) BOOL packable;
@@ -213,12 +238,28 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 /**
  * Describes a proto enum.
  **/
-@interface GPBEnumDescriptor : NSObject
+__attribute__((objc_subclassing_restricted))
+@interface GPBEnumDescriptor : NSObject<NSCopying>
+
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
 
 /** Name of the enum. */
 @property(nonatomic, readonly, copy) NSString *name;
 /** Function that validates that raw values are valid enum values. */
 @property(nonatomic, readonly) GPBEnumValidationFunc enumVerifier;
+/**
+ * Is this a closed enum, meaning that it:
+ * - Has a fixed set of named values.
+ * - Encountering values not in this set causes them to be treated as unknown
+ *   fields.
+ * - The first value (i.e., the default) may be nonzero.
+ *
+ * NOTE: This is only accurate if the generate sources for a proto file were
+ * generated with a protobuf release after the v21.9 version, as the ObjC
+ * generator wasn't capturing this information.
+ */
+@property(nonatomic, readonly) BOOL isClosed;
 
 /**
  * Returns the enum value name for the given raw enum.
@@ -293,7 +334,12 @@ typedef NS_ENUM(uint8_t, GPBFieldType) {
 /**
  * Describes a proto extension.
  **/
+__attribute__((objc_subclassing_restricted))
 @interface GPBExtensionDescriptor : NSObject<NSCopying>
+
+- (instancetype)init NS_UNAVAILABLE;
++ (instancetype)new NS_UNAVAILABLE;
+
 /** Field number under which the extension is stored. */
 @property(nonatomic, readonly) uint32_t fieldNumber;
 /** The containing message class, i.e. the class extended by this extension. */
