@@ -1,5 +1,3 @@
-#! /usr/bin/env python
-#
 # Protocol Buffers - Google's data interchange format
 # Copyright 2008 Google Inc.  All rights reserved.
 # https://developers.google.com/protocol-buffers/
@@ -35,13 +33,9 @@
 __author__ = 'matthewtoia@google.com (Matt Toia)'
 
 import copy
-import os
 import unittest
 import warnings
 
-from google.protobuf import unittest_import_pb2
-from google.protobuf import unittest_import_public_pb2
-from google.protobuf import unittest_pb2
 from google.protobuf import descriptor_pb2
 from google.protobuf.internal import api_implementation
 from google.protobuf.internal import descriptor_pool_test1_pb2
@@ -57,7 +51,9 @@ from google.protobuf import descriptor_database
 from google.protobuf import descriptor_pool
 from google.protobuf import message_factory
 from google.protobuf import symbol_database
-
+from google.protobuf import unittest_import_pb2
+from google.protobuf import unittest_import_public_pb2
+from google.protobuf import unittest_pb2
 
 
 warnings.simplefilter('error', DeprecationWarning)
@@ -245,10 +241,10 @@ class DescriptorPoolTestBase(object):
     self.assertRaises(KeyError, self.pool.FindMethodByName, '')
 
     # TODO(jieluo): Fix python to raise correct errors.
-    if api_implementation.Type() == 'cpp':
-      error_type = TypeError
-    else:
+    if api_implementation.Type() == 'python':
       error_type = AttributeError
+    else:
+      error_type = TypeError
     self.assertRaises(error_type, self.pool.FindMessageTypeByName, 0)
     self.assertRaises(error_type, self.pool.FindFieldByName, 0)
     self.assertRaises(error_type, self.pool.FindExtensionByName, 0)
@@ -397,16 +393,39 @@ class DescriptorPoolTestBase(object):
 
   def testAddSerializedFile(self):
     if isinstance(self, SecondaryDescriptorFromDescriptorDB):
-      if api_implementation.Type() == 'cpp':
+      if api_implementation.Type() != 'python':
         # Cpp extension cannot call Add on a DescriptorPool
         # that uses a DescriptorDatabase.
         # TODO(jieluo): Fix python and cpp extension diff.
         return
     self.pool = descriptor_pool.DescriptorPool()
-    self.pool.AddSerializedFile(self.factory_test1_fd.SerializeToString())
-    self.pool.AddSerializedFile(self.factory_test2_fd.SerializeToString())
+    file1 = self.pool.AddSerializedFile(
+        self.factory_test1_fd.SerializeToString())
+    file2 = self.pool.AddSerializedFile(
+        self.factory_test2_fd.SerializeToString())
+    self.assertEqual(file1.name,
+                     'google/protobuf/internal/factory_test1.proto')
+    self.assertEqual(file2.name,
+                     'google/protobuf/internal/factory_test2.proto')
     self.testFindMessageTypeByName()
+    file_json = self.pool.AddSerializedFile(
+        more_messages_pb2.DESCRIPTOR.serialized_pb)
+    field = file_json.message_types_by_name['class'].fields_by_name['int_field']
+    self.assertEqual(field.json_name, 'json_int')
 
+  def testAddSerializedFileTwice(self):
+    if isinstance(self, SecondaryDescriptorFromDescriptorDB):
+      if api_implementation.Type() != 'python':
+        # Cpp extension cannot call Add on a DescriptorPool
+        # that uses a DescriptorDatabase.
+        # TODO(jieluo): Fix python and cpp extension diff.
+        return
+    self.pool = descriptor_pool.DescriptorPool()
+    file1_first = self.pool.AddSerializedFile(
+        self.factory_test1_fd.SerializeToString())
+    file1_again = self.pool.AddSerializedFile(
+        self.factory_test1_fd.SerializeToString())
+    self.assertIs(file1_first, file1_again)
 
   def testEnumDefaultValue(self):
     """Test the default value of enums which don't start at zero."""
@@ -426,7 +445,7 @@ class DescriptorPoolTestBase(object):
     _CheckDefaultValue(file_descriptor)
 
     if isinstance(self, SecondaryDescriptorFromDescriptorDB):
-      if api_implementation.Type() == 'cpp':
+      if api_implementation.Type() != 'python':
         # Cpp extension cannot call Add on a DescriptorPool
         # that uses a DescriptorDatabase.
         # TODO(jieluo): Fix python and cpp extension diff.
@@ -480,7 +499,7 @@ class DescriptorPoolTestBase(object):
 
   def testAddFileDescriptor(self):
     if isinstance(self, SecondaryDescriptorFromDescriptorDB):
-      if api_implementation.Type() == 'cpp':
+      if api_implementation.Type() != 'python':
         # Cpp extension cannot call Add on a DescriptorPool
         # that uses a DescriptorDatabase.
         # TODO(jieluo): Fix python and cpp extension diff.
@@ -491,7 +510,7 @@ class DescriptorPoolTestBase(object):
 
   def testComplexNesting(self):
     if isinstance(self, SecondaryDescriptorFromDescriptorDB):
-      if api_implementation.Type() == 'cpp':
+      if api_implementation.Type() != 'python':
         # Cpp extension cannot call Add on a DescriptorPool
         # that uses a DescriptorDatabase.
         # TODO(jieluo): Fix python and cpp extension diff.
@@ -510,7 +529,7 @@ class DescriptorPoolTestBase(object):
 
   def testConflictRegister(self):
     if isinstance(self, SecondaryDescriptorFromDescriptorDB):
-      if api_implementation.Type() == 'cpp':
+      if api_implementation.Type() != 'python':
         # Cpp extension cannot call Add on a DescriptorPool
         # that uses a DescriptorDatabase.
         # TODO(jieluo): Fix python and cpp extension diff.
@@ -519,7 +538,7 @@ class DescriptorPoolTestBase(object):
         unittest_pb2.DESCRIPTOR.serialized_pb)
     conflict_fd = copy.deepcopy(unittest_fd)
     conflict_fd.name = 'other_file'
-    if api_implementation.Type() == 'cpp':
+    if api_implementation.Type() != 'python':
         pass
     else:
       pool = copy.deepcopy(self.pool)
@@ -533,7 +552,13 @@ class DescriptorPoolTestBase(object):
       pool._AddExtensionDescriptor(
           file_descriptor.extensions_by_name['optional_int32_extension'])
       pool.Add(unittest_fd)
-      pool.Add(conflict_fd)
+      with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        pool.Add(conflict_fd)
+        self.assertTrue(len(w))
+        self.assertIs(w[0].category, RuntimeWarning)
+        self.assertIn('Conflict register for file "other_file": ',
+                      str(w[0].message))
       pool.FindFileByName(unittest_fd.name)
       with self.assertRaises(TypeError):
         pool.FindFileByName(conflict_fd.name)
@@ -640,11 +665,11 @@ class SecondaryDescriptorFromDescriptorDB(DescriptorPoolTestBase,
     enum_value.number = 0
     self.db.Add(file_proto)
 
-    self.assertRaisesRegexp(KeyError, 'SubMessage',
-                            self.pool.FindMessageTypeByName,
-                            'collector.ErrorMessage')
-    self.assertRaisesRegexp(KeyError, 'SubMessage',
-                            self.pool.FindFileByName, 'error_file')
+    self.assertRaisesRegex(KeyError, 'SubMessage',
+                           self.pool.FindMessageTypeByName,
+                           'collector.ErrorMessage')
+    self.assertRaisesRegex(KeyError, 'SubMessage', self.pool.FindFileByName,
+                           'error_file')
     with self.assertRaises(KeyError) as exc:
       self.pool.FindFileByName('none_file')
     self.assertIn(str(exc.exception), ('\'none_file\'',
@@ -656,7 +681,7 @@ class SecondaryDescriptorFromDescriptorDB(DescriptorPoolTestBase,
     # called the first time, a KeyError will be raised but call the find
     # method later will return a descriptor which is not build.
     # TODO(jieluo): fix pure python to revert the load if file can not be build
-    if api_implementation.Type() == 'cpp':
+    if api_implementation.Type() != 'python':
       error_msg = ('Invalid proto descriptor for file "error_file":\\n  '
                    'collector.ErrorMessage.nested_message_field: "SubMessage" '
                    'is not defined.\\n  collector.ErrorMessage.MyOneof: Oneof '
@@ -892,8 +917,8 @@ class AddDescriptorTest(unittest.TestCase):
         pool.FindFileContainingSymbol(
             prefix + 'protobuf_unittest.TestAllTypes.NestedMessage').name)
 
-  @unittest.skipIf(api_implementation.Type() == 'cpp',
-                   'With the cpp implementation, Add() must be called first')
+  @unittest.skipIf(api_implementation.Type() != 'python',
+                   'Only pure python allows _Add*()')
   def testMessage(self):
     self._TestMessage('')
     self._TestMessage('.')
@@ -934,14 +959,14 @@ class AddDescriptorTest(unittest.TestCase):
         pool.FindFileContainingSymbol(
             prefix + 'protobuf_unittest.TestAllTypes.NestedEnum').name)
 
-  @unittest.skipIf(api_implementation.Type() == 'cpp',
-                   'With the cpp implementation, Add() must be called first')
+  @unittest.skipIf(api_implementation.Type() != 'python',
+                   'Only pure python allows _Add*()')
   def testEnum(self):
     self._TestEnum('')
     self._TestEnum('.')
 
-  @unittest.skipIf(api_implementation.Type() == 'cpp',
-                   'With the cpp implementation, Add() must be called first')
+  @unittest.skipIf(api_implementation.Type() != 'python',
+                   'Only pure python allows _Add*()')
   def testService(self):
     pool = descriptor_pool.DescriptorPool()
     with self.assertRaises(KeyError):
@@ -951,8 +976,8 @@ class AddDescriptorTest(unittest.TestCase):
         'protobuf_unittest.TestService',
         pool.FindServiceByName('protobuf_unittest.TestService').full_name)
 
-  @unittest.skipIf(api_implementation.Type() == 'cpp',
-                   'With the cpp implementation, Add() must be called first')
+  @unittest.skipIf(api_implementation.Type() != 'python',
+                   'Only pure python allows _Add*()')
   def testFile(self):
     pool = descriptor_pool.DescriptorPool()
     pool._AddFileDescriptor(unittest_pb2.DESCRIPTOR)
@@ -1029,7 +1054,7 @@ class AddDescriptorTest(unittest.TestCase):
 
   def testAddTypeError(self):
     pool = descriptor_pool.DescriptorPool()
-    if api_implementation.Type() == 'cpp':
+    if api_implementation.Type() != 'python':
       with self.assertRaises(TypeError):
         pool.AddDescriptor(0)
       with self.assertRaises(TypeError):
@@ -1101,34 +1126,51 @@ TEST1_FILE = ProtoFile(
 
 TEST2_FILE = ProtoFile(
     'google/protobuf/internal/descriptor_pool_test2.proto',
-    'google.protobuf.python.internal',
-    {
-        'DescriptorPoolTest3': MessageType({
-            'NestedEnum': EnumType([('NU', 13), ('XI', 14)]),
-            'NestedMessage': MessageType({
-                'NestedEnum': EnumType([('OMICRON', 15), ('PI', 16)]),
-                'DeepNestedMessage': MessageType({
-                    'NestedEnum': EnumType([('RHO', 17), ('SIGMA', 18)]),
+    'google.protobuf.python.internal', {
+        'DescriptorPoolTest3':
+            MessageType(
+                {
+                    'NestedEnum':
+                        EnumType([('NU', 13), ('XI', 14)]),
+                    'NestedMessage':
+                        MessageType(
+                            {
+                                'NestedEnum':
+                                    EnumType([('OMICRON', 15), ('PI', 16)]),
+                                'DeepNestedMessage':
+                                    MessageType(
+                                        {
+                                            'NestedEnum':
+                                                EnumType([('RHO', 17),
+                                                          ('SIGMA', 18)]),
+                                        }, [
+                                            ('nested_enum',
+                                             EnumField(1, 'NestedEnum', 'RHO')),
+                                            ('nested_field',
+                                             StringField(2, 'sigma')),
+                                        ]),
+                            }, [
+                                ('nested_enum', EnumField(
+                                    1, 'NestedEnum', 'PI')),
+                                ('nested_field', StringField(2, 'nu')),
+                                ('deep_nested_message',
+                                 MessageField(3, 'DeepNestedMessage')),
+                            ])
                 }, [
-                    ('nested_enum', EnumField(1, 'NestedEnum', 'RHO')),
-                    ('nested_field', StringField(2, 'sigma')),
+                    ('nested_enum', EnumField(1, 'NestedEnum', 'XI')),
+                    ('nested_message', MessageField(2, 'NestedMessage')),
+                ],
+                extensions=[
+                    ('descriptor_pool_test',
+                     ExtensionField(1001, 'DescriptorPoolTest1')),
                 ]),
-            }, [
-                ('nested_enum', EnumField(1, 'NestedEnum', 'PI')),
-                ('nested_field', StringField(2, 'nu')),
-                ('deep_nested_message', MessageField(3, 'DeepNestedMessage')),
-            ])
-        }, [
-            ('nested_enum', EnumField(1, 'NestedEnum', 'XI')),
-            ('nested_message', MessageField(2, 'NestedMessage')),
-        ], extensions=[
-            ('descriptor_pool_test',
-             ExtensionField(1001, 'DescriptorPoolTest1')),
-        ]),
     },
-    dependencies=['google/protobuf/internal/descriptor_pool_test1.proto',
-                  'google/protobuf/internal/more_messages.proto'],
-    public_dependencies=['google/protobuf/internal/more_messages.proto'])
+    dependencies=[
+        'google/protobuf/internal/more_messages.proto',
+        'google/protobuf/internal/descriptor_pool_test1.proto',
+    ],
+    public_dependencies=[
+        'google/protobuf/internal/more_messages.proto'])
 
 
 if __name__ == '__main__':
