@@ -1,21 +1,44 @@
 #! /usr/bin/env python
 # Protocol Buffers - Google's data interchange format
 # Copyright 2008 Google Inc.  All rights reserved.
+# https://developers.google.com/protocol-buffers/
 #
-# Use of this source code is governed by a BSD-style
-# license that can be found in the LICENSE file or at
-# https://developers.google.com/open-source/licenses/bsd
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#     * Redistributions of source code must retain the above copyright
+# notice, this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above
+# copyright notice, this list of conditions and the following disclaimer
+# in the documentation and/or other materials provided with the
+# distribution.
+#     * Neither the name of Google Inc. nor the names of its
+# contributors may be used to endorse or promote products derived from
+# this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 # See README for usage instructions.
 
 # pylint:disable=missing-module-docstring
 # pylint:disable=g-bad-import-order
+from distutils import util
 import fnmatch
 import glob
 import os
 import pkg_resources
 import re
-import shutil
 import subprocess
 import sys
 import sysconfig
@@ -23,10 +46,14 @@ import sysconfig
 # pylint:disable=g-importing-member
 # pylint:disable=g-multiple-import
 
+# We must use setuptools, not distutils, because we need to use the
+# namespace_packages option for the "google" package.
 from setuptools import setup, Extension, find_packages
 
-from setuptools.command.build_ext import build_ext as _build_ext
-from setuptools.command.build_py import build_py as _build_py
+from distutils.command.build_ext import build_ext as _build_ext
+from distutils.command.build_py import build_py as _build_py
+from distutils.command.clean import clean as _clean
+from distutils.spawn import find_executable
 
 # Find the Protocol Compiler.
 if 'PROTOC' in os.environ and os.path.exists(os.environ['PROTOC']):
@@ -35,16 +62,16 @@ elif os.path.exists('../bazel-bin/protoc'):
   protoc = '../bazel-bin/protoc'
 elif os.path.exists('../bazel-bin/protoc.exe'):
   protoc = '../bazel-bin/protoc.exe'
-elif os.path.exists('../protoc'):
+elif os.path.exists('protoc'):
   protoc = '../protoc'
-elif os.path.exists('../protoc.exe'):
+elif os.path.exists('protoc.exe'):
   protoc = '../protoc.exe'
 elif os.path.exists('../vsprojects/Debug/protoc.exe'):
   protoc = '../vsprojects/Debug/protoc.exe'
 elif os.path.exists('../vsprojects/Release/protoc.exe'):
   protoc = '../vsprojects/Release/protoc.exe'
 else:
-  protoc = shutil.which('protoc')
+  protoc = find_executable('protoc')
 
 
 def GetVersion():
@@ -104,6 +131,7 @@ def GenerateUnittestProtos():
   GenProto('../src/google/protobuf/map_unittest.proto', False)
   GenProto('../src/google/protobuf/test_messages_proto3.proto', False)
   GenProto('../src/google/protobuf/test_messages_proto2.proto', False)
+  GenProto('../src/google/protobuf/unittest_arena.proto', False)
   GenProto('../src/google/protobuf/unittest.proto', False)
   GenProto('../src/google/protobuf/unittest_custom_options.proto', False)
   GenProto('../src/google/protobuf/unittest_import.proto', False)
@@ -112,7 +140,6 @@ def GenerateUnittestProtos():
   GenProto('../src/google/protobuf/unittest_mset_wire_format.proto', False)
   GenProto('../src/google/protobuf/unittest_no_generic_services.proto', False)
   GenProto('../src/google/protobuf/unittest_proto3_arena.proto', False)
-  GenProto('../src/google/protobuf/unittest_retention.proto', False)
   GenProto('../src/google/protobuf/util/json_format.proto', False)
   GenProto('../src/google/protobuf/util/json_format_proto3.proto', False)
   GenProto('google/protobuf/internal/any_test.proto', False)
@@ -140,6 +167,21 @@ def GenerateUnittestProtos():
   GenProto('google/protobuf/pyext/python.proto', False)
 
 
+class CleanCmd(_clean):
+  """Custom clean command for building the protobuf extension."""
+
+  def run(self):
+    # Delete generated files in the code tree.
+    for (dirpath, unused_dirnames, filenames) in os.walk('.'):
+      for filename in filenames:
+        filepath = os.path.join(dirpath, filename)
+        if (filepath.endswith('_pb2.py') or filepath.endswith('.pyc') or
+            filepath.endswith('.so') or filepath.endswith('.o')):
+          os.remove(filepath)
+    # _clean is an old-style class, so super() doesn't work.
+    _clean.run(self)
+
+
 class BuildPyCmd(_build_py):
   """Custom build_py command for building the protobuf runtime."""
 
@@ -212,7 +254,7 @@ def GetOptionFromArgv(option_str):
 
 
 def _GetFlagValues(flag_long, flag_short):
-  """Searches sys.argv for setuptools-style flags and yields values."""
+  """Searches sys.argv for distutils-style flags and yields values."""
 
   expect_value = flag_long.endswith('=')
   flag_res = [re.compile(r'--?%s(=(.*))?' %
@@ -343,10 +385,8 @@ if __name__ == '__main__':
                          pkg_resources.parse_version('10.9.0')):
         os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.9'
         os.environ['_PYTHON_HOST_PLATFORM'] = re.sub(
-            r'macosx-[0-9]+\.[0-9]+-(.+)',
-            r'macosx-10.9-\1',
-            sysconfig.get_platform(),
-        )
+            r'macosx-[0-9]+\.[0-9]+-(.+)', r'macosx-10.9-\1',
+            util.get_platform())
 
     # https://github.com/Theano/Theano/issues/4926
     if sys.platform == 'win32':
@@ -408,28 +448,25 @@ if __name__ == '__main__':
       classifiers=[
           'Programming Language :: Python',
           'Programming Language :: Python :: 3',
-          # LINT.IfChange
-          # Remove importlib fallback path when we drop Python 3.8 support.
+          'Programming Language :: Python :: 3.7',
           'Programming Language :: Python :: 3.8',
-          # LINT.ThenChange(//depot/google3/google/protobuf/internal/test_util.py)
           'Programming Language :: Python :: 3.9',
           'Programming Language :: Python :: 3.10',
-          'Programming Language :: Python :: 3.11',
       ],
       namespace_packages=['google'],
       packages=find_packages(
           exclude=[
               'import_test_package',
               'protobuf_distutils',
-          ],
-      ),
+          ],),
       test_suite='google.protobuf.internal',
       cmdclass={
+          'clean': CleanCmd,
           'build_py': BuildPyCmd,
           'build_ext': BuildExtCmd,
           'test_conformance': TestConformanceCmd,
       },
       install_requires=install_requires,
       ext_modules=ext_module_list,
-      python_requires='>=3.8',
+      python_requires='>=3.7',
   )
