@@ -37,6 +37,7 @@
 #include <limits>
 #include <string>
 #include <typeinfo>
+#include <vector>
 
 #include "absl/base/attributes.h"
 #include "absl/synchronization/mutex.h"
@@ -156,6 +157,29 @@ SerialArena::SerialArena(FirstSerialArena, ArenaBlock* b,
   limit_ = b->Limit();
 }
 
+std::vector<void*> SerialArena::PeekCleanupListForTesting() {
+  std::vector<void*> res;
+
+  ArenaBlock* b = head();
+  if (b->IsSentry()) return res;
+
+  const auto peek_list = [&](const char* pos, const char* end) {
+    while (pos != end) {
+      pos += cleanup::PeekNode(pos, res);
+    }
+  };
+
+  peek_list(limit_, b->Limit());
+  for (b = b->next; b; b = b->next) {
+    peek_list(reinterpret_cast<char*>(b->cleanup_nodes), b->Limit());
+  }
+  return res;
+}
+
+std::vector<void*> ThreadSafeArena::PeekCleanupListForTesting() {
+  return GetSerialArena()->PeekCleanupListForTesting();
+}
+
 void SerialArena::Init(ArenaBlock* b, size_t offset) {
   set_ptr(b->Pointer(offset));
   limit_ = b->Limit();
@@ -197,18 +221,21 @@ void* SerialArena::AllocateAlignedFallback(size_t n) {
 
 PROTOBUF_NOINLINE
 void* SerialArena::AllocateFromStringBlockFallback() {
+  ABSL_DCHECK_EQ(string_block_unused_.load(std::memory_order_relaxed), 0U);
   if (string_block_) {
-    ABSL_DCHECK_EQ(string_block_unused_.load(std::memory_order_relaxed), 0U);
-    space_used_.store(space_used_.load(std::memory_order_relaxed) +
-                          string_block_->effective_size(),
-                      std::memory_order_relaxed);
+    AddSpaceUsed(string_block_->effective_size());
   }
 
-  string_block_ = StringBlock::New(string_block_);
-  space_allocated_.store(space_allocated_.load(std::memory_order_relaxed) +
-                             string_block_->allocated_size(),
-                         std::memory_order_relaxed);
-
+  void* ptr;
+  size_t size = StringBlock::NextSize(string_block_);
+  if (MaybeAllocateAligned(size, &ptr)) {
+    // Correct space_used_ to avoid double counting
+    AddSpaceUsed(-size);
+    string_block_ = StringBlock::Emplace(ptr, size, string_block_);
+  } else {
+    string_block_ = StringBlock::New(string_block_);
+    AddSpaceAllocated(string_block_->allocated_size());
+  }
   size_t unused = string_block_->effective_size() - sizeof(std::string);
   string_block_unused_.store(unused, std::memory_order_relaxed);
   return string_block_->AtOffset(unused);
@@ -240,8 +267,7 @@ void SerialArena::AllocateNewBlock(size_t n) {
     // Record how much used in this block.
     used = static_cast<size_t>(ptr() - old_head->Pointer(kBlockHeaderSize));
     wasted = old_head->size - used;
-    space_used_.store(space_used_.load(std::memory_order_relaxed) + used,
-                      std::memory_order_relaxed);
+    AddSpaceUsed(used);
   }
 
   // TODO(sbenza): Evaluate if pushing unused space into the cached blocks is a
@@ -253,9 +279,7 @@ void SerialArena::AllocateNewBlock(size_t n) {
   // We don't want to emit an expensive RMW instruction that requires
   // exclusive access to a cacheline. Hence we write it in terms of a
   // regular add.
-  space_allocated_.store(
-      space_allocated_.load(std::memory_order_relaxed) + mem.n,
-      std::memory_order_relaxed);
+  AddSpaceAllocated(mem.n);
   ThreadSafeArenaStats::RecordAllocateStats(parent_.arena_stats_.MutableStats(),
                                             /*used=*/used,
                                             /*allocated=*/mem.n, wasted);
@@ -278,19 +302,21 @@ uint64_t SerialArena::SpaceUsed() const {
   // usage of the *current* block.
   // TODO(mkruskal) Consider eliminating this race in exchange for a possible
   // performance hit on ARM (see cl/455186837).
-  uint64_t current_space_used =
-      string_block_ ? string_block_->effective_size() -
-                          string_block_unused_.load(std::memory_order_relaxed)
-                    : 0;
+
+  uint64_t space_used = 0;
+  if (string_block_) {
+    size_t unused = string_block_unused_.load(std::memory_order_relaxed);
+    space_used += string_block_->effective_size() - unused;
+  }
   const ArenaBlock* h = head_.load(std::memory_order_acquire);
-  if (h->IsSentry()) return current_space_used;
+  if (h->IsSentry()) return space_used;
 
   const uint64_t current_block_size = h->size;
-  current_space_used += std::min(
+  space_used += std::min(
       static_cast<uint64_t>(
           ptr() - const_cast<ArenaBlock*>(h)->Pointer(kBlockHeaderSize)),
       current_block_size);
-  return current_space_used + space_used_.load(std::memory_order_relaxed);
+  return space_used + space_used_.load(std::memory_order_relaxed);
 }
 
 size_t SerialArena::FreeStringBlocks(StringBlock* string_block,
@@ -493,7 +519,7 @@ ThreadSafeArena::ThreadCache& ThreadSafeArena::thread_cache() {
       new internal::ThreadLocalStorage<ThreadCache>();
   return *thread_cache_->Get();
 }
-#elif defined(PROTOBUF_USE_DLLS)
+#elif defined(PROTOBUF_USE_DLLS) && defined(_MSC_VER)
 ThreadSafeArena::ThreadCache& ThreadSafeArena::thread_cache() {
   static PROTOBUF_THREAD_LOCAL ThreadCache thread_cache;
   return thread_cache;
@@ -752,11 +778,15 @@ void* ThreadSafeArena::AllocateAlignedWithCleanup(size_t n, size_t align,
 }
 
 void ThreadSafeArena::AddCleanup(void* elem, void (*cleanup)(void*)) {
+  GetSerialArena()->AddCleanup(elem, cleanup);
+}
+
+SerialArena* ThreadSafeArena::GetSerialArena() {
   SerialArena* arena;
   if (PROTOBUF_PREDICT_FALSE(!GetSerialArenaFast(&arena))) {
     arena = GetSerialArenaFallback(kMaxCleanupNodeSize);
   }
-  arena->AddCleanup(elem, cleanup);
+  return arena;
 }
 
 PROTOBUF_NOINLINE
@@ -768,11 +798,7 @@ void* ThreadSafeArena::AllocateAlignedWithCleanupFallback(
 
 PROTOBUF_NOINLINE
 void* ThreadSafeArena::AllocateFromStringBlock() {
-  SerialArena* arena;
-  if (PROTOBUF_PREDICT_FALSE(!GetSerialArenaFast(&arena))) {
-    arena = GetSerialArenaFallback(0);
-  }
-  return arena->AllocateFromStringBlock();
+  return GetSerialArena()->AllocateFromStringBlock();
 }
 
 template <typename Functor>
@@ -906,6 +932,10 @@ void* Arena::AllocateAlignedWithCleanup(size_t n, size_t align,
   return impl_.AllocateAlignedWithCleanup(n, align, destructor);
 }
 
+std::vector<void*> Arena::PeekCleanupListForTesting() {
+  return impl_.PeekCleanupListForTesting();
+}
+
 }  // namespace protobuf
 }  // namespace google
 
