@@ -1,9 +1,32 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
+// https://developers.google.com/protocol-buffers/
 //
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file or at
-// https://developers.google.com/open-source/licenses/bsd
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+//     * Redistributions of source code must retain the above copyright
+// notice, this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above
+// copyright notice, this list of conditions and the following disclaimer
+// in the documentation and/or other materials provided with the
+// distribution.
+//     * Neither the name of Google Inc. nor the names of its
+// contributors may be used to endorse or promote products derived from
+// this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // This file defines an Arena allocator for better allocation performance.
 
@@ -11,7 +34,6 @@
 #define GOOGLE_PROTOBUF_ARENA_H__
 
 #include <limits>
-#include <string>
 #include <type_traits>
 #include <utility>
 #include <vector>
@@ -26,8 +48,9 @@ using type_info = ::type_info;
 #include <typeinfo>
 #endif
 
-#include "absl/log/absl_check.h"
+#include "absl/meta/type_traits.h"
 #include "google/protobuf/arena_align.h"
+#include "google/protobuf/arena_config.h"
 #include "google/protobuf/port.h"
 #include "google/protobuf/serial_arena.h"
 #include "google/protobuf/thread_safe_arena.h"
@@ -99,7 +122,7 @@ struct ArenaOptions {
   // individual arena allocation request occurs with a size larger than this
   // maximum). Requested block sizes increase up to this value, then remain
   // here.
-  size_t max_block_size = internal::AllocationPolicy::kDefaultMaxBlockSize;
+  size_t max_block_size = internal::GetDefaultArenaMaxBlockSize();
 
   // An initial block of memory for the arena to use, or nullptr for none. If
   // provided, the block must live at least as long as the arena itself. The
@@ -367,7 +390,7 @@ class PROTOBUF_EXPORT PROTOBUF_ALIGNAS(8) Arena final {
   // message, or nullptr otherwise. If possible, the call resolves at compile
   // time. Note that we can often devirtualize calls to `value->GetArena()` so
   // usually calling this method is unnecessary.
-  // TODO: remove this function.
+  // TODO(b/271599886): remove this function.
   template <typename T>
   ABSL_DEPRECATED(
       "This will be removed in a future release. Call value->GetArena() "
@@ -389,7 +412,8 @@ class PROTOBUF_EXPORT PROTOBUF_ALIGNAS(8) Arena final {
     // mutually exclusive fashion, we use implicit conversions to base classes
     // to force an explicit ranking for our preferences.  The lowest ranked
     // version that compiles will be accepted.
-    struct Rank1 {};
+    struct Rank2 {};
+    struct Rank1 : Rank2 {};
     struct Rank0 : Rank1 {};
 
     static Arena* GetOwningArena(const T* p) {
@@ -409,16 +433,31 @@ class PROTOBUF_EXPORT PROTOBUF_ALIGNAS(8) Arena final {
 
     static void InternalSwap(T* a, T* b) { a->InternalSwap(b); }
 
-    static Arena* GetArena(T* p) { return GetArena(Rank0{}, p); }
+    static Arena* GetArenaForAllocation(T* p) {
+      return GetArenaForAllocation(Rank0{}, p);
+    }
+
+    static Arena* GetArena(T* p) {
+      // Rather than replicate probing for `GetArena` with fallback to nullptr,
+      // we borrow the implementation of `GetArenaForAllocation` but skip
+      // `Rank0` which probes for `GetArenaForAllocation`.
+      return GetArenaForAllocation(Rank1{}, p);
+    }
 
     template <typename U>
-    static auto GetArena(Rank0, U* p)
+    static auto GetArenaForAllocation(Rank0, U* p)
+        -> EnableIfArena<decltype(p->GetArenaForAllocation())> {
+      return p->GetArenaForAllocation();
+    }
+
+    template <typename U>
+    static auto GetArenaForAllocation(Rank1, U* p)
         -> EnableIfArena<decltype(p->GetArena())> {
       return p->GetArena();
     }
 
     template <typename U>
-    static Arena* GetArena(Rank1, U*) {
+    static Arena* GetArenaForAllocation(Rank2, U*) {
       return nullptr;
     }
 
@@ -461,26 +500,15 @@ class PROTOBUF_EXPORT PROTOBUF_ALIGNAS(8) Arena final {
   // Provides access to protected GetOwningArena to generated messages.  For
   // internal use only.
   template <typename T>
-  static Arena* InternalGetOwningArena(T* p) {
-    ABSL_DCHECK_EQ(InternalHelper<T>::GetOwningArena(p),
-                   InternalHelper<T>::GetArena(p));
+  static Arena* InternalGetOwningArena(const T* p) {
     return InternalHelper<T>::GetOwningArena(p);
   }
 
-  // Wraps InternalGetArena() and will be removed soon.
+  // Provides access to protected GetArenaForAllocation to generated messages.
   // For internal use only.
   template <typename T>
   static Arena* InternalGetArenaForAllocation(T* p) {
-    return InternalHelper<T>::GetArena(p);
-  }
-
-  // Provides access to protected GetArena to generated messages.
-  // For internal use only.
-  template <typename T>
-  static Arena* InternalGetArena(T* p) {
-    ABSL_DCHECK_EQ(InternalHelper<T>::GetOwningArena(p),
-                   InternalHelper<T>::GetArena(p));
-    return InternalHelper<T>::GetArena(p);
+    return InternalHelper<T>::GetArenaForAllocation(p);
   }
 
   // Helper typetraits that indicates support for arenas in a type T at compile
@@ -543,9 +571,7 @@ class PROTOBUF_EXPORT PROTOBUF_ALIGNAS(8) Arena final {
     if (trivial) {
       return AllocateAligned(sizeof(T), alignof(T));
     } else {
-      // We avoid instantiating arena_destruct_object<T> in the trivial case.
-      constexpr auto dtor = &internal::cleanup::arena_destruct_object<
-          std::conditional_t<trivial, std::string, T>>;
+      constexpr auto dtor = &internal::cleanup::arena_destruct_object<T>;
       return AllocateAlignedWithCleanup(sizeof(T), alignof(T), dtor);
     }
   }
