@@ -41,10 +41,12 @@
 #include <string>
 #include <utility>
 #include <vector>
-#include <google/protobuf/stubs/common.h>
+
+#include "absl/strings/string_view.h"
+#include "google/protobuf/port.h"
 
 // Must be included last.
-#include <google/protobuf/port_def.inc>
+#include "google/protobuf/port_def.inc"
 
 namespace google {
 namespace protobuf {
@@ -69,7 +71,9 @@ class GeneratorContext;
 // be registered with CommandLineInterface to support various languages.
 class PROTOC_EXPORT CodeGenerator {
  public:
-  inline CodeGenerator() {}
+  CodeGenerator() {}
+  CodeGenerator(const CodeGenerator&) = delete;
+  CodeGenerator& operator=(const CodeGenerator&) = delete;
   virtual ~CodeGenerator();
 
   // Generates code for the given proto file, generating one or more files in
@@ -120,9 +124,6 @@ class PROTOC_EXPORT CodeGenerator {
   // version of the library. When protobufs does a api breaking change, the
   // method can be removed.
   virtual bool HasGenerateAll() const { return true; }
-
- private:
-  GOOGLE_DISALLOW_EVIL_CONSTRUCTORS(CodeGenerator);
 };
 
 // CodeGenerators generate one or more files in a given directory.  This
@@ -131,8 +132,10 @@ class PROTOC_EXPORT CodeGenerator {
 // runs.
 class PROTOC_EXPORT GeneratorContext {
  public:
-  inline GeneratorContext() {
+  GeneratorContext() {
   }
+  GeneratorContext(const GeneratorContext&) = delete;
+  GeneratorContext& operator=(const GeneratorContext&) = delete;
   virtual ~GeneratorContext();
 
   // Opens the given file, truncating it if it exists, and returns a
@@ -177,9 +180,6 @@ class PROTOC_EXPORT GeneratorContext {
   // this GeneratorContext.
   virtual void GetCompilerVersion(Version* version) const;
 
-
- private:
-  GOOGLE_DISALLOW_EVIL_CONSTRUCTORS(GeneratorContext);
 };
 
 // The type GeneratorContext was once called OutputDirectory. This typedef
@@ -189,19 +189,19 @@ typedef GeneratorContext OutputDirectory;
 // Several code generators treat the parameter argument as holding a
 // list of options separated by commas.  This helper function parses
 // a set of comma-delimited name/value pairs: e.g.,
-//   "foo=bar,baz,qux=corge"
+//   "foo=bar,baz,moo=corge"
 // parses to the pairs:
-//   ("foo", "bar"), ("baz", ""), ("qux", "corge")
+//   ("foo", "bar"), ("baz", ""), ("moo", "corge")
 PROTOC_EXPORT void ParseGeneratorParameter(
-    const std::string&, std::vector<std::pair<std::string, std::string> >*);
+    absl::string_view, std::vector<std::pair<std::string, std::string> >*);
 
 // Strips ".proto" or ".protodevel" from the end of a filename.
-PROTOC_EXPORT std::string StripProto(const std::string& filename);
+PROTOC_EXPORT std::string StripProto(absl::string_view filename);
 
 }  // namespace compiler
 }  // namespace protobuf
 }  // namespace google
 
-#include <google/protobuf/port_undef.inc>
+#include "google/protobuf/port_undef.inc"
 
 #endif  // GOOGLE_PROTOBUF_COMPILER_CODE_GENERATOR_H__
