@@ -52,6 +52,7 @@ namespace protobuf {
 namespace compiler {
 namespace cpp {
 namespace {
+
 std::string NumberedCcFileName(absl::string_view basename, int number) {
   return absl::StrCat(basename, ".out/", number, ".cc");
 }
@@ -161,6 +162,14 @@ bool CppGenerator::Generate(const FileDescriptor* file,
     } else if (key == "proto_static_reflection_h") {
     } else if (key == "annotate_accessor") {
       file_options.annotate_accessor = true;
+    } else if (key == "protos_for_field_listener_events") {
+      for (absl::string_view proto : absl::StrSplit(value, ':')) {
+        if (proto == file->name()) {
+          file_options.field_listener_options.inject_field_listener_events =
+              true;
+          break;
+        }
+      }
     } else if (key == "inject_field_listener_events") {
       file_options.field_listener_options.inject_field_listener_events = true;
     } else if (key == "forbidden_field_listener_events") {
@@ -182,8 +191,6 @@ bool CppGenerator::Generate(const FileDescriptor* file,
     } else if (key == "experimental_tail_call_table_mode") {
       if (value == "never") {
         file_options.tctable_mode = Options::kTCTableNever;
-      } else if (value == "guarded") {
-        file_options.tctable_mode = Options::kTCTableGuarded;
       } else if (value == "always") {
         file_options.tctable_mode = Options::kTCTableAlways;
       } else {
