@@ -55,13 +55,22 @@ using internal::cpp::Utf8CheckMode;
 using google::protobuf::internal::WireFormat;
 using google::protobuf::internal::WireFormatLite;
 
+bool HasWeakFields(const Descriptor* descriptor) {
+  for (int i = 0; i < descriptor->field_count(); i++) {
+    if (descriptor->field(i)->options().weak()) {
+      return true;
+    }
+  }
+  return false;
+}
 bool UseDirectTcParserTable(const FieldDescriptor* field,
                             const Options& options) {
   if (field->cpp_type() != field->CPPTYPE_MESSAGE) return false;
   auto* m = field->message_type();
   return !m->options().message_set_wire_format() &&
          m->file()->options().optimize_for() != FileOptions::CODE_SIZE &&
-         !HasSimpleBaseClass(m, options) && !HasTracker(m, options)
+         !HasSimpleBaseClass(m, options) && !HasTracker(m, options) &&
+         !HasWeakFields(m)
       ;  // NOLINT(whitespace/semicolon)
 }
 
@@ -90,13 +99,23 @@ class ParseFunctionGenerator::GeneratedOptionProvider final
   explicit GeneratedOptionProvider(ParseFunctionGenerator* gen) : gen_(gen) {}
   TailCallTableInfo::PerFieldOptions GetForField(
       const FieldDescriptor* field) const final {
-    return {IsLazy(field, gen_->options_, gen_->scc_analyzer_),
-            IsStringInlined(field, gen_->options_),
-            IsImplicitWeakField(field, gen_->options_, gen_->scc_analyzer_),
-            UseDirectTcParserTable(field, gen_->options_),
-            GetOptimizeFor(field->file(), gen_->options_) ==
-                FileOptions::LITE_RUNTIME,
-            ShouldSplit(field, gen_->options_)};
+    const auto verify_flag = [&] {
+      if (IsEagerlyVerifiedLazy(field, gen_->options_, gen_->scc_analyzer_))
+        return internal::field_layout::kTvEager;
+      if (IsLazilyVerifiedLazy(field, gen_->options_))
+        return internal::field_layout::kTvLazy;
+      return internal::field_layout::TransformValidation{};
+    };
+    return {
+        verify_flag(),
+        IsStringInlined(field, gen_->options_),
+        IsImplicitWeakField(field, gen_->options_, gen_->scc_analyzer_),
+        UseDirectTcParserTable(field, gen_->options_),
+        GetOptimizeFor(field->file(), gen_->options_) ==
+            FileOptions::LITE_RUNTIME,
+        ShouldSplit(field, gen_->options_),
+        /* uses_codegen */ true,
+    };
   }
 
  private:
@@ -127,21 +146,12 @@ ParseFunctionGenerator::ParseFunctionGenerator(
 }
 
 void ParseFunctionGenerator::GenerateMethodDecls(io::Printer* printer) {
-  Formatter format(printer, variables_);
-  if (should_generate_tctable()) {
-    format.Outdent();
-    if (should_generate_guarded_tctable()) {
-      format("#ifdef PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-    }
-    format(
-        " private:\n"
-        "  static const char* Tct_ParseFallback(PROTOBUF_TC_PARAM_DECL);\n"
-        " public:\n");
-    if (should_generate_guarded_tctable()) {
-      format("#endif\n");
-    }
-    format.Indent();
+  if (HasWeakFields(descriptor_)) {
+    // We use the reflection based one.
+    ABSL_CHECK(HasDescriptorMethods(descriptor_->file(), options_));
+    return;
   }
+  Formatter format(printer, variables_);
   format(
       "const char* _InternalParse(const char* ptr, "
       "::$proto_ns$::internal::ParseContext* ctx) final;\n");
@@ -167,35 +177,27 @@ void ParseFunctionGenerator::GenerateMethodImpls(io::Printer* printer) {
         "      internal_default_instance(), &_internal_metadata_, ctx);\n"
         "}\n");
   }
+  if (HasWeakFields(descriptor_)) {
+    // We use the reflection based one.
+    ABSL_CHECK(HasDescriptorMethods(descriptor_->file(), options_));
+    need_parse_function = false;
+  }
   if (!should_generate_tctable()) {
     if (need_parse_function) {
       GenerateLoopingParseFunction(format);
     }
     return;
   }
-  if (should_generate_guarded_tctable()) {
-    format("#ifdef PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n\n");
-  }
   if (need_parse_function) {
     GenerateTailcallParseFunction(format);
   }
-  if (tc_table_info_->use_generated_fallback) {
-    GenerateTailcallFallbackFunction(format);
-  }
-  if (should_generate_guarded_tctable()) {
-    if (need_parse_function) {
-      format("\n#else  // PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n\n");
-      GenerateLoopingParseFunction(format);
-    }
-    format("\n#endif  // PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-  }
 }
 
 bool ParseFunctionGenerator::should_generate_tctable() const {
   if (options_.tctable_mode == Options::kTCTableNever) {
     return false;
   }
-  if (HasSimpleBaseClass(descriptor_, options_)) {
+  if (HasSimpleBaseClass(descriptor_, options_) || HasWeakFields(descriptor_)) {
     return false;
   }
   return true;
@@ -216,64 +218,6 @@ void ParseFunctionGenerator::GenerateTailcallParseFunction(Formatter& format) {
       "}\n\n");
 }
 
-static bool NeedsUnknownEnumSupport(const Descriptor* descriptor) {
-  for (int i = 0; i < descriptor->field_count(); ++i) {
-    auto* field = descriptor->field(i);
-    if (field->is_repeated() && field->cpp_type() == field->CPPTYPE_ENUM &&
-        !internal::cpp::HasPreservingUnknownEnumSemantics(field)) {
-      return true;
-    }
-  }
-  return false;
-}
-
-void ParseFunctionGenerator::GenerateTailcallFallbackFunction(
-    Formatter& format) {
-  ABSL_CHECK(should_generate_tctable());
-  format(
-      "const char* $classname$::Tct_ParseFallback(PROTOBUF_TC_PARAM_DECL) {\n"
-      "#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) return nullptr\n");
-  format.Indent();
-  format("auto* typed_msg = static_cast<$classname$*>(msg);\n");
-
-  // If we need a side channel, generate the check to jump to the generic
-  // handler to deal with the side channel data.
-  if (NeedsUnknownEnumSupport(descriptor_)) {
-    format(
-        "if (PROTOBUF_PREDICT_FALSE(\n"
-        "    _pbi::TcParser::MustFallbackToGeneric(PROTOBUF_TC_PARAM_PASS))) "
-        "{\n"
-        "  PROTOBUF_MUSTTAIL return "
-        "::_pbi::TcParser::GenericFallback$1$(PROTOBUF_TC_PARAM_PASS);\n"
-        "}\n",
-        GetOptimizeFor(descriptor_->file(), options_) ==
-                FileOptions::LITE_RUNTIME
-            ? "Lite"
-            : "");
-  }
-
-  if (num_hasbits_ > 0) {
-    // Sync hasbits
-    format("typed_msg->_impl_._has_bits_[0] |= hasbits;\n");
-  }
-  format("::uint32_t tag = data.tag();\n");
-
-  format.Set("msg", "typed_msg->");
-  format.Set("this", "typed_msg");
-  format.Set("has_bits", "typed_msg->_impl_._has_bits_");
-  format.Set("next_tag", "goto next_tag");
-  GenerateParseIterationBody(format, descriptor_,
-                             tc_table_info_->fallback_fields);
-
-  format.Outdent();
-  format(
-      "next_tag:\n"
-      "message_done:\n"
-      "  return ptr;\n"
-      "#undef CHK_\n"
-      "}\n");
-}
-
 struct SkipEntry16 {
   uint16_t skipmap;
   uint16_t field_entry_offset;
@@ -310,11 +254,6 @@ void ParseFunctionGenerator::GenerateDataDecls(io::Printer* printer) {
     return;
   }
   Formatter format(printer, variables_);
-  if (should_generate_guarded_tctable()) {
-    format.Outdent();
-    format("#ifdef PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-    format.Indent();
-  }
   auto field_num_to_entry_table = MakeNumToEntryTable(ordered_fields_);
   format(
       "friend class ::$proto_ns$::internal::TcParser;\n"
@@ -324,11 +263,6 @@ void ParseFunctionGenerator::GenerateDataDecls(io::Printer* printer) {
       tc_table_info_->aux_entries.size(),
       FieldNameDataSize(tc_table_info_->field_name_data),
       field_num_to_entry_table.size16());
-  if (should_generate_guarded_tctable()) {
-    format.Outdent();
-    format("#endif  // PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-    format.Indent();
-  }
 }
 
 void ParseFunctionGenerator::GenerateDataDefinitions(io::Printer* printer) {
@@ -336,13 +270,7 @@ void ParseFunctionGenerator::GenerateDataDefinitions(io::Printer* printer) {
     return;
   }
   Formatter format(printer, variables_);
-  if (should_generate_guarded_tctable()) {
-    format("#ifdef PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-  }
   GenerateTailCallTable(format);
-  if (should_generate_guarded_tctable()) {
-    format("#endif  // PROTOBUF_TAIL_CALL_TABLE_PARSER_ENABLED\n");
-  }
 }
 
 void ParseFunctionGenerator::GenerateLoopingParseFunction(Formatter& format) {
@@ -456,15 +384,10 @@ static NumToEntryTable MakeNumToEntryTable(
 void ParseFunctionGenerator::GenerateTailCallTable(Formatter& format) {
   ABSL_CHECK(should_generate_tctable());
   // All entries without a fast-path parsing function need a fallback.
-  std::string fallback;
-  if (tc_table_info_->use_generated_fallback) {
-    fallback = absl::StrCat(ClassName(descriptor_), "::Tct_ParseFallback");
-  } else {
-    fallback = "::_pbi::TcParser::GenericFallback";
-    if (GetOptimizeFor(descriptor_->file(), options_) ==
-        FileOptions::LITE_RUNTIME) {
-      absl::StrAppend(&fallback, "Lite");
-    }
+  std::string fallback = "::_pbi::TcParser::GenericFallback";
+  if (GetOptimizeFor(descriptor_->file(), options_) ==
+      FileOptions::LITE_RUNTIME) {
+    absl::StrAppend(&fallback, "Lite");
   }
 
   // For simplicity and speed, the table is not covering all proto
@@ -494,14 +417,10 @@ void ParseFunctionGenerator::GenerateTailCallTable(Formatter& format) {
       } else {
         format("0,  // no _has_bits_\n");
       }
-      if (descriptor_->extension_range_count() == 1) {
-        format(
-            "PROTOBUF_FIELD_OFFSET($classname$, $extensions$),\n"
-            "$1$, $2$,  // extension_range_{low,high}\n",
-            descriptor_->extension_range(0)->start,
-            descriptor_->extension_range(0)->end);
+      if (descriptor_->extension_range_count() != 0) {
+        format("PROTOBUF_FIELD_OFFSET($classname$, $extensions$),\n");
       } else {
-        format("0, 0, 0,  // no _extensions_\n");
+        format("0, // no _extensions_\n");
       }
       format("$1$, $2$,  // max_field_number, fast_idx_mask\n",
              (ordered_fields_.empty() ? 0 : ordered_fields_.back()->number()),
@@ -622,6 +541,11 @@ void ParseFunctionGenerator::GenerateTailCallTable(Formatter& format) {
                        QualifiedDefaultInstancePtr(
                            aux_entry.field->message_type(), options_));
                 break;
+              case TailCallTableInfo::kMessageVerifyFunc:
+                format("{$1$::InternalVerify},\n",
+                       QualifiedClassName(aux_entry.field->message_type(),
+                                          options_));
+                break;
               case TailCallTableInfo::kEnumRange:
                 format("{$1$, $2$},\n", aux_entry.enum_range.start,
                        aux_entry.enum_range.size);
@@ -634,6 +558,29 @@ void ParseFunctionGenerator::GenerateTailCallTable(Formatter& format) {
               case TailCallTableInfo::kNumericOffset:
                 format("{_fl::Offset{$1$}},\n", aux_entry.offset);
                 break;
+              case TailCallTableInfo::kMapAuxInfo: {
+                auto utf8_check = internal::cpp::GetUtf8CheckMode(
+                    aux_entry.field,
+                    GetOptimizeFor(aux_entry.field->file(), options_) ==
+                        FileOptions::LITE_RUNTIME);
+                auto* map_value = aux_entry.field->message_type()->map_value();
+                const bool validated_enum =
+                    map_value->type() == FieldDescriptor::TYPE_ENUM &&
+                    !internal::cpp::HasPreservingUnknownEnumSemantics(
+                        map_value);
+                format(
+                    "{::_pbi::TcParser::GetMapAuxInfo<decltype($classname$("
+                    ").$1$)>($2$, $3$, $4$)},\n",
+                    FieldMemberName(aux_entry.field, /*split=*/false),
+                    utf8_check == internal::cpp::Utf8CheckMode::kStrict,
+                    utf8_check == internal::cpp::Utf8CheckMode::kVerify,
+                    validated_enum);
+                break;
+              }
+              case TailCallTableInfo::kCreateInArena:
+                format("{::_pbi::TcParser::CreateInArenaStorageCb<$1$>},\n",
+                       QualifiedClassName(aux_entry.desc, options_));
+                break;
             }
           }
         }
@@ -757,13 +704,16 @@ static void FormatFieldKind(Formatter& format,
         format(" | ::_fl::kRep$1$", rep);
       }
 
-      static constexpr const char* kXFormNames[] = {nullptr, "Default", "Table",
-                                                    "WeakPtr"};
+      static constexpr const char* kXFormNames[2][4] = {
+          {nullptr, "Default", "Table", "WeakPtr"}, {nullptr, "Eager", "Lazy"}};
+
       static_assert((fl::kTvDefault >> fl::kTvShift) == 1, "");
       static_assert((fl::kTvTable >> fl::kTvShift) == 2, "");
       static_assert((fl::kTvWeakPtr >> fl::kTvShift) == 3, "");
+      static_assert((fl::kTvEager >> fl::kTvShift) == 1, "");
+      static_assert((fl::kTvLazy >> fl::kTvShift) == 2, "");
 
-      if (auto* xform = kXFormNames[tv_index]) {
+      if (auto* xform = kXFormNames[rep_index == 2][tv_index]) {
         format(" | ::_fl::kTv$1$", xform);
       }
       break;
@@ -928,10 +878,17 @@ void ParseFunctionGenerator::GenerateStrings(Formatter& format,
                                              const FieldDescriptor* field,
                                              bool check_utf8) {
   FieldOptions::CType ctype = FieldOptions::STRING;
-  if (!options_.opensource_runtime) {
-    // Open source doesn't support other ctypes;
-    ctype = field->options().ctype();
+
+  if (!field->is_repeated() && field->type() == FieldDescriptor::TYPE_BYTES &&
+      field->options().ctype() == FieldOptions::CORD) {
+    ctype = FieldOptions::CORD;
+  } else {
+    if (!options_.opensource_runtime) {
+      // Open source doesn't support other ctypes;
+      ctype = field->options().ctype();
+    }
   }
+
   if (!field->is_repeated() && !options_.opensource_runtime &&
       GetOptimizeFor(field->file(), options_) != FileOptions::LITE_RUNTIME &&
       // For now only use arena string for strings with empty defaults.
@@ -1028,7 +985,7 @@ void ParseFunctionGenerator::GenerateLengthDelim(Formatter& format,
           const FieldDescriptor* val = field->message_type()->map_value();
           ABSL_CHECK(val);
           if (val->type() == FieldDescriptor::TYPE_ENUM &&
-              !internal::cpp::HasPreservingUnknownEnumSemantics(field)) {
+              !internal::cpp::HasPreservingUnknownEnumSemantics(val)) {
             format(
                 "auto object = "
                 "::$proto_ns$::internal::InitEnumParseWrapper<"
@@ -1151,7 +1108,7 @@ void ParseFunctionGenerator::GenerateFieldBody(
         format.Set("enum_type",
                    QualifiedClassName(field->enum_type(), options_));
         format(
-            "$uint32$ val = ::$proto_ns$::internal::ReadVarint32(&ptr);\n"
+            "$int32$ val = ::$proto_ns$::internal::ReadVarint32(&ptr);\n"
             "CHK_(ptr);\n");
         if (!internal::cpp::HasPreservingUnknownEnumSemantics(field)) {
           format(
