@@ -42,6 +42,7 @@
 #include "google/protobuf/compiler/java/doc_comment.h"
 #include "google/protobuf/compiler/java/helpers.h"
 #include "google/protobuf/compiler/java/name_resolver.h"
+#include "google/protobuf/descriptor_legacy.h"
 #include "google/protobuf/io/printer.h"
 #include "google/protobuf/wire_format.h"
 
@@ -327,17 +328,19 @@ void ImmutableMessageFieldGenerator::GenerateBuilderMembers(
 
   // Message.Builder clearField()
   WriteFieldDocComment(printer, descriptor_);
-  printer->Print(variables_,
-                 "$deprecation$public Builder clear$capitalized_name$() {\n"
-                 "  $clear_has_field_bit_builder$\n"
-                 "  $name$_ = null;\n"
-                 "  if ($name$Builder_ != null) {\n"
-                 "    $name$Builder_.dispose();\n"
-                 "    $name$Builder_ = null;\n"
-                 "  }\n"
-                 "  $on_changed$\n"
-                 "  return this;\n"
-                 "}\n");
+  printer->Print(
+      variables_,
+      "$deprecation$public Builder ${$clear$capitalized_name$$}$() {\n"
+      "  $clear_has_field_bit_builder$\n"
+      "  $name$_ = null;\n"
+      "  if ($name$Builder_ != null) {\n"
+      "    $name$Builder_.dispose();\n"
+      "    $name$Builder_ = null;\n"
+      "  }\n"
+      "  $on_changed$\n"
+      "  return this;\n"
+      "}\n");
+  printer->Annotate("{", "}", descriptor_);
 
   // Field.Builder getFieldBuilder()
   WriteFieldDocComment(printer, descriptor_);
@@ -414,7 +417,7 @@ void ImmutableMessageFieldGenerator::GenerateKotlinDslMembers(
 }
 
 void ImmutableMessageFieldGenerator::GenerateKotlinOrNull(io::Printer* printer) const {
-  if (descriptor_->has_optional_keyword()) {
+  if (FieldDescriptorLegacy(descriptor_).has_optional_keyword()) {
     printer->Print(variables_,
                    "public val $classname$Kt.Dsl.$name$OrNull: $kt_type$?\n"
                    "  get() = $kt_dsl_builder$.$name$OrNull\n");
