@@ -41,8 +41,6 @@
 //
 // This header covers RepeatedPtrField.
 
-// IWYU pragma: private, include "net/proto2/public/repeated_field.h"
-
 #ifndef GOOGLE_PROTOBUF_REPEATED_PTR_FIELD_H__
 #define GOOGLE_PROTOBUF_REPEATED_PTR_FIELD_H__
 
@@ -58,15 +56,15 @@
 #include <string>
 #include <type_traits>
 
-#include <google/protobuf/stubs/logging.h>
-#include <google/protobuf/stubs/common.h>
-#include <google/protobuf/arena.h>
-#include <google/protobuf/port.h>
-#include <google/protobuf/message_lite.h>
+#include "google/protobuf/arena.h"
+#include "google/protobuf/port.h"
+#include "absl/log/absl_check.h"
+#include "google/protobuf/message_lite.h"
+#include "google/protobuf/port.h"
 
 
 // Must be included last.
-#include <google/protobuf/port_def.inc>
+#include "google/protobuf/port_def.inc"
 
 #ifdef SWIG
 #error "You cannot SWIG proto headers"
@@ -187,22 +185,22 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
 
   template <typename TypeHandler>
   const typename TypeHandler::Type& at(int index) const {
-    GOOGLE_CHECK_GE(index, 0);
-    GOOGLE_CHECK_LT(index, current_size_);
+    ABSL_CHECK_GE(index, 0);
+    ABSL_CHECK_LT(index, current_size_);
     return *cast<TypeHandler>(rep_->elements[index]);
   }
 
   template <typename TypeHandler>
   typename TypeHandler::Type& at(int index) {
-    GOOGLE_CHECK_GE(index, 0);
-    GOOGLE_CHECK_LT(index, current_size_);
+    ABSL_CHECK_GE(index, 0);
+    ABSL_CHECK_LT(index, current_size_);
     return *cast<TypeHandler>(rep_->elements[index]);
   }
 
   template <typename TypeHandler>
   typename TypeHandler::Type* Mutable(int index) {
-    GOOGLE_DCHECK_GE(index, 0);
-    GOOGLE_DCHECK_LT(index, current_size_);
+    ABSL_DCHECK_GE(index, 0);
+    ABSL_DCHECK_LT(index, current_size_);
     return cast<TypeHandler>(rep_->elements[index]);
   }
 
@@ -210,7 +208,8 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   typename TypeHandler::Type* Add(
       const typename TypeHandler::Type* prototype = nullptr) {
     if (rep_ != nullptr && current_size_ < rep_->allocated_size) {
-      return cast<TypeHandler>(rep_->elements[current_size_++]);
+      return cast<TypeHandler>(
+          rep_->elements[ExchangeCurrentSize(current_size_ + 1)]);
     }
     typename TypeHandler::Type* result =
         TypeHandler::NewFromPrototype(prototype, arena_);
@@ -223,7 +222,9 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
       typename std::enable_if<TypeHandler::Movable::value>::type* = nullptr>
   inline void Add(typename TypeHandler::Type&& value) {
     if (rep_ != nullptr && current_size_ < rep_->allocated_size) {
-      *cast<TypeHandler>(rep_->elements[current_size_++]) = std::move(value);
+      *cast<TypeHandler>(
+          rep_->elements[ExchangeCurrentSize(current_size_ + 1)]) =
+          std::move(value);
       return;
     }
     if (!rep_ || rep_->allocated_size == total_size_) {
@@ -232,13 +233,13 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     ++rep_->allocated_size;
     typename TypeHandler::Type* result =
         TypeHandler::New(arena_, std::move(value));
-    rep_->elements[current_size_++] = result;
+    rep_->elements[ExchangeCurrentSize(current_size_ + 1)] = result;
   }
 
   template <typename TypeHandler>
   void Delete(int index) {
-    GOOGLE_DCHECK_GE(index, 0);
-    GOOGLE_DCHECK_LT(index, current_size_);
+    ABSL_DCHECK_GE(index, 0);
+    ABSL_DCHECK_LT(index, current_size_);
     TypeHandler::Delete(cast<TypeHandler>(rep_->elements[index]), arena_);
   }
 
@@ -267,8 +268,8 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
 
   template <typename TypeHandler>
   const typename TypeHandler::Type& Get(int index) const {
-    GOOGLE_DCHECK_GE(index, 0);
-    GOOGLE_DCHECK_LT(index, current_size_);
+    ABSL_DCHECK_GE(index, 0);
+    ABSL_DCHECK_LT(index, current_size_);
     return *cast<TypeHandler>(rep_->elements[index]);
   }
 
@@ -281,14 +282,9 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   template <typename TypeHandler>
   void Clear() {
     const int n = current_size_;
-    GOOGLE_DCHECK_GE(n, 0);
+    ABSL_DCHECK_GE(n, 0);
     if (n > 0) {
-      void* const* elements = rep_->elements;
-      int i = 0;
-      do {
-        TypeHandler::Clear(cast<TypeHandler>(elements[i++]));
-      } while (i < n);
-      current_size_ = 0;
+      ClearNonEmpty<TypeHandler>();
     }
   }
 
@@ -300,14 +296,14 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     // binary. This calls a shared implementation with most of the logic,
     // passing a function pointer to another type-specific piece of code that
     // calls the object-allocate and merge handlers.
-    GOOGLE_DCHECK_NE(&other, this);
+    ABSL_DCHECK_NE(&other, this);
     if (other.current_size_ == 0) return;
     MergeFromInternal(other,
                       &RepeatedPtrFieldBase::MergeFromInnerLoop<TypeHandler>);
   }
 
   inline void InternalSwap(RepeatedPtrFieldBase* rhs) {
-    GOOGLE_DCHECK(this != rhs);
+    ABSL_DCHECK(this != rhs);
 
     // Swap all fields at once.
     auto temp = std::make_tuple(rhs->arena_, rhs->current_size_,
@@ -317,11 +313,40 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     std::tie(arena_, current_size_, total_size_, rep_) = temp;
   }
 
+  // Prepares the container for adding elements via `AddAllocatedForParse`.
+  // It ensures some invariants to avoid checking then in the Add loop:
+  //  - rep_ is not null.
+  //  - there are no preallocated elements.
+  //  Returns true if the invariants hold and `AddAllocatedForParse` can be
+  //  used.
+  bool PrepareForParse() {
+    if (current_size_ == total_size_) {
+      InternalExtend(1);
+    }
+    return rep_->allocated_size == current_size_;
+  }
+
+  // Similar to `AddAllocated` but faster.
+  // Can only be invoked after a call to `PrepareForParse` that returned `true`,
+  // or other calls to `AddAllocatedForParse`.
+  template <typename TypeHandler>
+  void AddAllocatedForParse(typename TypeHandler::Type* value) {
+    PROTOBUF_ASSUME(rep_ != nullptr);
+    PROTOBUF_ASSUME(current_size_ == rep_->allocated_size);
+    if (current_size_ == total_size_) {
+      // The array is completely full with no cleared objects, so grow it.
+      InternalExtend(1);
+    }
+    rep_->elements[current_size_++] = value;
+    ++rep_->allocated_size;
+  }
+
  protected:
   template <typename TypeHandler>
   void RemoveLast() {
-    GOOGLE_DCHECK_GT(current_size_, 0);
-    TypeHandler::Clear(cast<TypeHandler>(rep_->elements[--current_size_]));
+    ABSL_DCHECK_GT(current_size_, 0);
+    ExchangeCurrentSize(current_size_ - 1);
+    TypeHandler::Clear(cast<TypeHandler>(rep_->elements[current_size_]));
   }
 
   template <typename TypeHandler>
@@ -403,7 +428,8 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   template <typename TypeHandler>
   typename TypeHandler::Type* AddFromCleared() {
     if (rep_ != nullptr && current_size_ < rep_->allocated_size) {
-      return cast<TypeHandler>(rep_->elements[current_size_++]);
+      return cast<TypeHandler>(
+          rep_->elements[ExchangeCurrentSize(current_size_ + 1)]);
     } else {
       return nullptr;
     }
@@ -439,7 +465,7 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
       ++rep_->allocated_size;
     }
 
-    rep_->elements[current_size_++] = value;
+    rep_->elements[ExchangeCurrentSize(current_size_ + 1)] = value;
   }
 
   template <typename TypeHandler>
@@ -453,9 +479,10 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   // arena.
   template <typename TypeHandler>
   typename TypeHandler::Type* UnsafeArenaReleaseLast() {
-    GOOGLE_DCHECK_GT(current_size_, 0);
+    ABSL_DCHECK_GT(current_size_, 0);
+    ExchangeCurrentSize(current_size_ - 1);
     typename TypeHandler::Type* result =
-        cast<TypeHandler>(rep_->elements[--current_size_]);
+        cast<TypeHandler>(rep_->elements[current_size_]);
     --rep_->allocated_size;
     if (current_size_ < rep_->allocated_size) {
       // There are cleared elements on the end; replace the removed element
@@ -471,9 +498,10 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
 
   template <typename TypeHandler>
   void AddCleared(typename TypeHandler::Type* value) {
-    GOOGLE_DCHECK(GetOwningArena() == nullptr) << "AddCleared() can only be used on a "
-                                           "RepeatedPtrField not on an arena.";
-    GOOGLE_DCHECK(TypeHandler::GetOwningArena(value) == nullptr)
+    ABSL_DCHECK(GetOwningArena() == nullptr)
+        << "AddCleared() can only be used on a "
+           "RepeatedPtrField not on an arena.";
+    ABSL_DCHECK(TypeHandler::GetOwningArena(value) == nullptr)
         << "AddCleared() can only accept values not on an arena.";
     if (!rep_ || rep_->allocated_size == total_size_) {
       Reserve(total_size_ + 1);
@@ -483,12 +511,11 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
 
   template <typename TypeHandler>
   PROTOBUF_NODISCARD typename TypeHandler::Type* ReleaseCleared() {
-    GOOGLE_DCHECK(GetOwningArena() == nullptr)
+    ABSL_DCHECK(GetOwningArena() == nullptr)
         << "ReleaseCleared() can only be used on a RepeatedPtrField not on "
         << "an arena.";
-    GOOGLE_DCHECK(GetOwningArena() == nullptr);
-    GOOGLE_DCHECK(rep_ != nullptr);
-    GOOGLE_DCHECK_GT(rep_->allocated_size, current_size_);
+    ABSL_DCHECK(rep_ != nullptr);
+    ABSL_DCHECK_GT(rep_->allocated_size, current_size_);
     return cast<TypeHandler>(rep_->elements[--rep_->allocated_size]);
   }
 
@@ -508,8 +535,7 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
         // allocated list.
         elems[rep_->allocated_size] = elems[current_size_];
       }
-      elems[current_size_] = value;
-      current_size_ = current_size_ + 1;
+      elems[ExchangeCurrentSize(current_size_ + 1)] = value;
       rep_->allocated_size = rep_->allocated_size + 1;
     } else {
       AddAllocatedSlowWithCopy<TypeHandler>(value, element_arena, arena);
@@ -531,8 +557,7 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
         // allocated list.
         elems[rep_->allocated_size] = elems[current_size_];
       }
-      elems[current_size_] = value;
-      current_size_ = current_size_ + 1;
+      elems[ExchangeCurrentSize(current_size_ + 1)] = value;
       ++rep_->allocated_size;
     } else {
       UnsafeArenaAddAllocated<TypeHandler>(value);
@@ -582,10 +607,10 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   template <typename TypeHandler>
   typename TypeHandler::Type* ReleaseLastInternal(std::false_type) {
     // ReleaseLast() for types that *do not* implement merge/copy behavior --
-    // this is the same as UnsafeArenaReleaseLast(). Note that we GOOGLE_DCHECK-fail if
-    // we're on an arena, since the user really should implement the copy
-    // operation in this case.
-    GOOGLE_DCHECK(GetOwningArena() == nullptr)
+    // this is the same as UnsafeArenaReleaseLast(). Note that we
+    // ABSL_DCHECK-fail if we're on an arena, since the user really should
+    // implement the copy operation in this case.
+    ABSL_DCHECK(GetOwningArena() == nullptr)
         << "ReleaseLast() called on a RepeatedPtrField that is on an arena, "
         << "with a type that does not implement MergeFrom. This is unsafe; "
         << "please implement MergeFrom for your type.";
@@ -595,10 +620,10 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   template <typename TypeHandler>
   PROTOBUF_NOINLINE void SwapFallback(RepeatedPtrFieldBase* other) {
 #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
-    GOOGLE_DCHECK(GetOwningArena() == nullptr ||
-           other->GetOwningArena() != GetOwningArena());
+    ABSL_DCHECK(GetOwningArena() == nullptr ||
+                other->GetOwningArena() != GetOwningArena());
 #else   // PROTOBUF_FORCE_COPY_IN_SWAP
-    GOOGLE_DCHECK(other->GetOwningArena() != GetOwningArena());
+    ABSL_DCHECK(other->GetOwningArena() != GetOwningArena());
 #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
 
     // Copy semantics in this case. We try to improve efficiency by placing the
@@ -612,7 +637,8 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     temp.Destroy<TypeHandler>();  // Frees rep_ if `other` had no arena.
   }
 
-  inline Arena* GetArena() const { return arena_; }
+  // Gets the Arena on which this RepeatedPtrField stores its elements.
+  inline Arena* GetArena() const { return GetOwningArena(); }
 
  protected:
   inline Arena* GetOwningArena() const { return arena_; }
@@ -635,6 +661,16 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
   Arena* arena_;
   int current_size_;
   int total_size_;
+
+  // Replaces current_size_ with new_size and returns the previous value of
+  // current_size_. This function is intended to be the only place where
+  // current_size_ is modified.
+  inline int ExchangeCurrentSize(int new_size) {
+    int prev_size = current_size_;
+    current_size_ = new_size;
+    return prev_size;
+  }
+
   struct Rep {
     int allocated_size;
     // Here we declare a huge array as a way of approximating C's "flexible
@@ -654,12 +690,27 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     return reinterpret_cast<const typename TypeHandler::Type*>(element);
   }
 
+  // Out-of-line helper routine for Clear() once the inlined check has
+  // determined the container is non-empty
+  template <typename TypeHandler>
+  PROTOBUF_NOINLINE void ClearNonEmpty() {
+    const int n = current_size_;
+    void* const* elements = rep_->elements;
+    int i = 0;
+    ABSL_DCHECK_GT(
+        n,
+        0);  // do/while loop to avoid initial test because we know n > 0
+    do {
+      TypeHandler::Clear(cast<TypeHandler>(elements[i++]));
+    } while (i < n);
+    ExchangeCurrentSize(0);
+  }
+
   // Non-templated inner function to avoid code duplication. Takes a function
   // pointer to the type-specific (templated) inner allocate/merge loop.
-  void MergeFromInternal(const RepeatedPtrFieldBase& other,
-                         void (RepeatedPtrFieldBase::*inner_loop)(void**,
-                                                                  void**, int,
-                                                                  int)) {
+  PROTOBUF_NOINLINE void MergeFromInternal(
+      const RepeatedPtrFieldBase& other,
+      void (RepeatedPtrFieldBase::*inner_loop)(void**, void**, int, int)) {
     // Note: wrapper has already guaranteed that other.rep_ != nullptr here.
     int other_size = other.current_size_;
     void** other_elements = other.rep_->elements;
@@ -667,7 +718,7 @@ class PROTOBUF_EXPORT RepeatedPtrFieldBase {
     int allocated_elems = rep_->allocated_size - current_size_;
     (this->*inner_loop)(new_elements, other_elements, other_size,
                         allocated_elems);
-    current_size_ += other_size;
+    ExchangeCurrentSize(current_size_ + other_size);
     if (rep_->allocated_size < current_size_) {
       rep_->allocated_size = current_size_;
     }
@@ -762,7 +813,7 @@ class GenericTypeHandler {
     }
   }
   static inline Arena* GetOwningArena(GenericType* value) {
-    return Arena::GetOwningArena<Type>(value);
+    return Arena::InternalGetOwningArena(value);
   }
 
   static inline void Clear(GenericType* value) { value->Clear(); }
@@ -820,10 +871,10 @@ class StringTypeHandler {
   typedef std::string Type;
   using Movable = IsMovable<Type>;
 
-  static inline std::string* New(Arena* arena) {
+  static PROTOBUF_NOINLINE std::string* New(Arena* arena) {
     return Arena::Create<std::string>(arena);
   }
-  static inline std::string* New(Arena* arena, std::string&& value) {
+  static PROTOBUF_NOINLINE std::string* New(Arena* arena, std::string&& value) {
     return Arena::Create<std::string>(arena, std::move(value));
   }
   static inline std::string* NewFromPrototype(const std::string*,
@@ -851,6 +902,21 @@ class StringTypeHandler {
 // Messages.
 template <typename Element>
 class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
+  static_assert(!std::is_const<Element>::value,
+                "We do not support const value types.");
+  static_assert(!std::is_volatile<Element>::value,
+                "We do not support volatile value types.");
+  static_assert(!std::is_pointer<Element>::value,
+                "We do not support pointer value types.");
+  static_assert(!std::is_reference<Element>::value,
+                "We do not support reference value types.");
+  static constexpr PROTOBUF_ALWAYS_INLINE void StaticValidityCheck() {
+    static_assert(
+        absl::disjunction<
+            internal::is_supported_string_type<Element>,
+            internal::is_supported_message_type<Element>>::value,
+        "We only support string and Message types in RepeatedPtrField.");
+  }
 
  public:
   constexpr RepeatedPtrField();
@@ -934,11 +1000,8 @@ class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
 
   // Gets the underlying array.  This pointer is possibly invalidated by
   // any add or remove operation.
-  //
-  // This API is deprecated. Instead of directly working with element array,
-  // use APIs in repeated_field_util.h; e.g. sorting, etc.
-  PROTOBUF_DEPRECATED_MSG("Use APIs in repeated_field_util.h")
-  Element** mutable_data();
+  Element**
+  mutable_data();
   const Element* const* data() const;
 
   // Swaps entire contents with "other". If they are on separate arenas, then
@@ -948,7 +1011,7 @@ class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
   // Swaps entire contents with "other". Caller should guarantee that either
   // both fields are on the same arena or both are on the heap. Swapping between
   // different arenas with this function is disallowed and is caught via
-  // GOOGLE_DCHECK.
+  // ABSL_DCHECK.
   void UnsafeArenaSwap(RepeatedPtrField* other);
 
   // Swaps two elements.
@@ -1082,23 +1145,28 @@ class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
 
   // Gets the number of cleared objects that are currently being kept
   // around for reuse.
+  ABSL_DEPRECATED("This will be removed in a future release")
   int ClearedCount() const;
-#ifndef PROTOBUF_FUTURE_BREAKING_CHANGES
+
+#ifndef PROTOBUF_FUTURE_REMOVE_CLEARED_API
   // Adds an element to the pool of cleared objects, passing ownership to
   // the RepeatedPtrField.  The element must be cleared prior to calling
   // this method.
   //
   // This method cannot be called when either the repeated field or |value| is
-  // on an arena; both cases will trigger a GOOGLE_DCHECK-failure.
+  // on an arena; both cases will trigger a ABSL_DCHECK-failure.
+  ABSL_DEPRECATED("This will be removed in a future release")
   void AddCleared(Element* value);
   // Removes and returns a single element from the cleared pool, passing
   // ownership to the caller.  The element is guaranteed to be cleared.
   // Requires:  ClearedCount() > 0
   //
   // This method cannot be called when the repeated field is on an arena; doing
-  // so will trigger a GOOGLE_DCHECK-failure.
-  PROTOBUF_NODISCARD Element* ReleaseCleared();
-#endif  // !PROTOBUF_FUTURE_BREAKING_CHANGES
+  // so will trigger a ABSL_DCHECK-failure.
+  PROTOBUF_NODISCARD
+  ABSL_DEPRECATED("This will be removed in a future release")
+  Element* ReleaseCleared();
+#endif  // !PROTOBUF_FUTURE_REMOVE_CLEARED_API
 
   // Removes the element referenced by position.
   //
@@ -1116,7 +1184,12 @@ class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
   iterator erase(const_iterator first, const_iterator last);
 
   // Gets the arena on which this RepeatedPtrField stores its elements.
+  inline Arena* GetArena();
+
+#ifndef PROTOBUF_FUTURE_REMOVE_CONST_REPEATEDFIELD_GETARENA_API
+  ABSL_DEPRECATED("This will be removed in a future release")
   inline Arena* GetArena() const;
+#endif  // !PROTOBUF_FUTURE_REMOVE_CONST_REPEATEDFIELD_GETARENA_API
 
   // For internal use only.
   //
@@ -1141,7 +1214,12 @@ class RepeatedPtrField final : private internal::RepeatedPtrFieldBase {
   void ExtractSubrangeInternal(int start, int num, Element** elements,
                                std::false_type);
 
+  void AddAllocatedForParse(Element* p) {
+    return RepeatedPtrFieldBase::AddAllocatedForParse<TypeHandler>(p);
+  }
+
   friend class Arena;
+  friend class internal::TcParser;
 
   template <typename T>
   friend struct WeakRepeatedPtrField;
@@ -1162,27 +1240,34 @@ class RepeatedPtrField<std::string>::TypeHandler
 
 template <typename Element>
 constexpr RepeatedPtrField<Element>::RepeatedPtrField()
-    : RepeatedPtrFieldBase() {}
+    : RepeatedPtrFieldBase() {
+  StaticValidityCheck();
+}
 
 template <typename Element>
 inline RepeatedPtrField<Element>::RepeatedPtrField(Arena* arena)
-    : RepeatedPtrFieldBase(arena) {}
+    : RepeatedPtrFieldBase(arena) {
+  StaticValidityCheck();
+}
 
 template <typename Element>
 inline RepeatedPtrField<Element>::RepeatedPtrField(
     const RepeatedPtrField& other)
     : RepeatedPtrFieldBase() {
+  StaticValidityCheck();
   MergeFrom(other);
 }
 
 template <typename Element>
 template <typename Iter, typename>
 inline RepeatedPtrField<Element>::RepeatedPtrField(Iter begin, Iter end) {
+  StaticValidityCheck();
   Add(begin, end);
 }
 
 template <typename Element>
 RepeatedPtrField<Element>::~RepeatedPtrField() {
+  StaticValidityCheck();
 #ifdef __cpp_if_constexpr
   if constexpr (std::is_base_of<MessageLite, Element>::value) {
 #else
@@ -1270,7 +1355,7 @@ inline Element* RepeatedPtrField<Element>::Mutable(int index) {
 }
 
 template <typename Element>
-inline Element* RepeatedPtrField<Element>::Add() {
+PROTOBUF_NOINLINE Element* RepeatedPtrField<Element>::Add() {
   return RepeatedPtrFieldBase::Add<TypeHandler>();
 }
 
@@ -1285,7 +1370,7 @@ inline void RepeatedPtrField<Element>::Add(Iter begin, Iter end) {
   if (std::is_base_of<
           std::forward_iterator_tag,
           typename std::iterator_traits<Iter>::iterator_category>::value) {
-    int reserve = std::distance(begin, end);
+    int reserve = static_cast<int>(std::distance(begin, end));
     Reserve(size() + reserve);
   }
   for (; begin != end; ++begin) {
@@ -1300,9 +1385,9 @@ inline void RepeatedPtrField<Element>::RemoveLast() {
 
 template <typename Element>
 inline void RepeatedPtrField<Element>::DeleteSubrange(int start, int num) {
-  GOOGLE_DCHECK_GE(start, 0);
-  GOOGLE_DCHECK_GE(num, 0);
-  GOOGLE_DCHECK_LE(start + num, size());
+  ABSL_DCHECK_GE(start, 0);
+  ABSL_DCHECK_GE(num, 0);
+  ABSL_DCHECK_LE(start + num, size());
   for (int i = 0; i < num; ++i) {
     RepeatedPtrFieldBase::Delete<TypeHandler>(start + i);
   }
@@ -1322,13 +1407,13 @@ inline void RepeatedPtrField<Element>::ExtractSubrange(int start, int num,
 template <typename Element>
 inline void RepeatedPtrField<Element>::ExtractSubrangeInternal(
     int start, int num, Element** elements, std::true_type) {
-  GOOGLE_DCHECK_GE(start, 0);
-  GOOGLE_DCHECK_GE(num, 0);
-  GOOGLE_DCHECK_LE(start + num, size());
+  ABSL_DCHECK_GE(start, 0);
+  ABSL_DCHECK_GE(num, 0);
+  ABSL_DCHECK_LE(start + num, size());
 
   if (num == 0) return;
 
-  GOOGLE_DCHECK_NE(elements, nullptr)
+  ABSL_DCHECK_NE(elements, nullptr)
       << "Releasing elements without transferring ownership is an unsafe "
          "operation.  Use UnsafeArenaExtractSubrange.";
   if (elements == nullptr) {
@@ -1372,9 +1457,9 @@ inline void RepeatedPtrField<Element>::ExtractSubrangeInternal(
     int start, int num, Element** elements, std::false_type) {
   // This case is identical to UnsafeArenaExtractSubrange(). However, since
   // ExtractSubrange() must return heap-allocated objects by contract, and we
-  // cannot fulfill this contract if we are an on arena, we must GOOGLE_DCHECK() that
-  // we are not on an arena.
-  GOOGLE_DCHECK(GetOwningArena() == nullptr)
+  // cannot fulfill this contract if we are an on arena, we must ABSL_DCHECK()
+  // that we are not on an arena.
+  ABSL_DCHECK(GetOwningArena() == nullptr)
       << "ExtractSubrange() when arena is non-nullptr is only supported when "
       << "the Element type supplies a MergeFrom() operation to make copies.";
   UnsafeArenaExtractSubrange(start, num, elements);
@@ -1383,9 +1468,9 @@ inline void RepeatedPtrField<Element>::ExtractSubrangeInternal(
 template <typename Element>
 inline void RepeatedPtrField<Element>::UnsafeArenaExtractSubrange(
     int start, int num, Element** elements) {
-  GOOGLE_DCHECK_GE(start, 0);
-  GOOGLE_DCHECK_GE(num, 0);
-  GOOGLE_DCHECK_LE(start + num, size());
+  ABSL_DCHECK_GE(start, 0);
+  ABSL_DCHECK_GE(num, 0);
+  ABSL_DCHECK_LE(start + num, size());
 
   if (num > 0) {
     // Save the values of the removed elements if requested.
@@ -1430,8 +1515,8 @@ RepeatedPtrField<Element>::erase(const_iterator position) {
 template <typename Element>
 inline typename RepeatedPtrField<Element>::iterator
 RepeatedPtrField<Element>::erase(const_iterator first, const_iterator last) {
-  size_type pos_offset = std::distance(cbegin(), first);
-  size_type last_offset = std::distance(cbegin(), last);
+  size_type pos_offset = static_cast<size_type>(std::distance(cbegin(), first));
+  size_type last_offset = static_cast<size_type>(std::distance(cbegin(), last));
   DeleteSubrange(pos_offset, last_offset - pos_offset);
   return begin() + pos_offset;
 }
@@ -1456,7 +1541,7 @@ template <typename Element>
 inline void RepeatedPtrField<Element>::UnsafeArenaSwap(
     RepeatedPtrField* other) {
   if (this == other) return;
-  GOOGLE_DCHECK_EQ(GetOwningArena(), other->GetOwningArena());
+  ABSL_DCHECK_EQ(GetOwningArena(), other->GetOwningArena());
   RepeatedPtrFieldBase::InternalSwap(other);
 }
 
@@ -1465,10 +1550,17 @@ inline void RepeatedPtrField<Element>::SwapElements(int index1, int index2) {
   RepeatedPtrFieldBase::SwapElements(index1, index2);
 }
 
+template <typename Element>
+inline Arena* RepeatedPtrField<Element>::GetArena() {
+  return RepeatedPtrFieldBase::GetArena();
+}
+
+#ifndef PROTOBUF_FUTURE_REMOVE_CONST_REPEATEDFIELD_GETARENA_API
 template <typename Element>
 inline Arena* RepeatedPtrField<Element>::GetArena() const {
   return RepeatedPtrFieldBase::GetArena();
 }
+#endif  // !PROTOBUF_FUTURE_REMOVE_CONST_REPEATEDFIELD_GETARENA_API
 
 template <typename Element>
 inline Arena* RepeatedPtrField<Element>::GetOwningArena() const {
@@ -1505,7 +1597,7 @@ inline int RepeatedPtrField<Element>::ClearedCount() const {
   return RepeatedPtrFieldBase::ClearedCount();
 }
 
-#ifndef PROTOBUF_FUTURE_BREAKING_CHANGES
+#ifndef PROTOBUF_FUTURE_REMOVE_CLEARED_API
 template <typename Element>
 inline void RepeatedPtrField<Element>::AddCleared(Element* value) {
   return RepeatedPtrFieldBase::AddCleared<TypeHandler>(value);
@@ -1515,7 +1607,7 @@ template <typename Element>
 inline Element* RepeatedPtrField<Element>::ReleaseCleared() {
   return RepeatedPtrFieldBase::ReleaseCleared<TypeHandler>();
 }
-#endif  // !PROTOBUF_FUTURE_BREAKING_CHANGES
+#endif  // !PROTOBUF_FUTURE_REMOVE_CLEARED_API
 
 template <typename Element>
 inline void RepeatedPtrField<Element>::Reserve(int new_size) {
@@ -1959,12 +2051,9 @@ UnsafeArenaAllocatedRepeatedPtrFieldBackInserter(
       mutable_field);
 }
 
-extern template class PROTOBUF_EXPORT_TEMPLATE_DECLARE
-    RepeatedPtrField<std::string>;
-
 }  // namespace protobuf
 }  // namespace google
 
-#include <google/protobuf/port_undef.inc>
+#include "google/protobuf/port_undef.inc"
 
 #endif  // GOOGLE_PROTOBUF_REPEATED_PTR_FIELD_H__
